/*
 * A resource offering support for creating processes.
 *
 * Copyright (C) 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <systypes/fcntl.h>

#include "process_creating.h"
#include "process_creator_resource.h"

#include "notifier_client.h"
#include "process_creator_server.h"



/* Support for creating processes. */

ProcessCreatorResource::ProcessCreatorResource(const char *rm_filename)
: _creating(rm_filename)
{
}

ProcessCreatorResource::~ProcessCreatorResource()
{
}

ipc_server_default_config_type ProcessCreatorResource::config()
{
    return config_ProcessCreator;
}



/* ProcessCreator interface methods. */

long ProcessCreatorResource::start(l4_cap_idx_t program, l4_cap_idx_t *process)
{
    file_t file;

    file_init(&file);
    file.ref = program;

    /* Obtain a distinct and usable file object. */

    file_t *program_file = client_reopen(&file, O_RDONLY);

    /* NOTE: To be replaced with the actual arguments, perhaps involving a
             context object. */

    const char *argv[] = {"<program>"};

    /* Start the new process, obtaining a reference to it. */

    return _creating.start(program_file, 1, argv, process);
}

// vim: tabstop=4 expandtab shiftwidth=4
